/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *      Matthias Hardt   <mattmax@morphine.tv>                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MMSMUSICDAO_H_
#define MMSMUSICDAO_H_

#include "mmsdata/mmsmusicdata.h"

class MMSMusicDAO {

    private:
        IMMSDB *dbConnection;

        void setMMSDBConnection(IMMSDB *connection);
        IMMSDB *getMMSDBConnection();
        
        void insert(MMSMusicData *data);
        void update(MMSMusicData *data);

        MMSMusicData moveRecordToData(MMSRecordSet &rs);
        
        unsigned int getArtistID(const string artist, const bool update = true);
        unsigned int getAlbumID(const string album, const unsigned int trackCount = 0, const string cover = "", const bool update = true);
        unsigned int getFormatID(const string format, const bool update = true);
        unsigned int getGenreID(const string genre, const bool update = true);
        unsigned int getYearID(const string year, const bool update = true);

    public:
        MMSMusicDAO(IMMSDB *connection);

        void save(MMSMusicData *data);
        void save(vector<MMSMusicData *> dataList);

        void getStatistics(unsigned int &numSongs, unsigned int &numArtists, unsigned int &numAlbums, unsigned int &numGenres);
        void getStatistics(unsigned int &numSongs, unsigned int &numArtists, unsigned int &numAlbums, unsigned int &numGenres, string &lastPlayed);
        void setLastPlayed(const string lastPlayed);
        vector <MMSMusicData *> getSongs();
        vector <MMSMusicData *> getSongsByArtist(string artist);
        vector <MMSMusicData *> getSongsByAlbum(string album);
        vector <MMSMusicData *> getSongsByGenre(string genre);
        MMSMusicData * getSongByFilename(string filename);
        vector <string *> getArtists();
        vector <string *> getAlbums();
        vector <string *> getGenres();       
};

#endif /*MMSMUSICDAO_H_*/
