/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *      Matthias Hardt   <mattmax@morphine.tv>                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MMSPHOTODAO_H_
#define MMSPHOTODAO_H_

#include "mmsdata/mmsphotodata.h"
#include "mmstools/mmstools.h"

class MMSPhotoDAO {
    private:
        IMMSDB	*dbConnection;
   
        void setMMSDBConnection(IMMSDB *connection);
        IMMSDB *getMMSDBConnection();
        
        void insert(MMSPhotoData *data);
        void update(MMSPhotoData *data);

        MMSPhotoData moveRecordToData(MMSRecordSet &rs);
        
        unsigned int getPlaceID(const string album, const bool update = true);
        unsigned int getCameraID(const string album, const bool update = true);
        unsigned int getAlbumID(const string album, const string albumCover = "", const bool update = true);
        unsigned int getFormatID(const string format, const bool update = true);

    public:
        MMSPhotoDAO(IMMSDB *connection);

        void save(MMSPhotoData *data);
        void save(vector<MMSPhotoData *> dataList);

        void getStatistics(unsigned int &numPhotos, unsigned int &numPlaces, unsigned int &numAlbums);
        void getStatistics(unsigned int &numPhotos, unsigned int &numPlaces, unsigned int &numAlbums, string &lastViewed);
        void setLastViewed(const string lastViewed);
        vector <MMSPhotoData *> getPhotos();
        vector <MMSPhotoData *> getPhotosByAlbum(const string album);
        vector <MMSPhotoData *> getPhotosByPlace(const string place);
        MMSPhotoData * getPhotoByFilename(const string filename);
        vector < pair<string, string> > getAlbums();
        vector <string> getPlaces();
        bool albumExists(const string album);
};

#endif /*MMSPHOTODAO_H_*/
