/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MMSIMAGE_H_
#define MMSIMAGE_H_

#include "mmsgui/mmswidget.h"
#include "mmsgui/mmsimagethread.h"

//! With this class you can display pictures and animations.
/*!
All widgets derived from the base widget class can display pictures at its background.
This image class additionally can display pictures over the background.
As special feature the image class can play animations. Currently the GIF format will be supported.   
The image widget cannot be focused.
\author Jens Schneider
*/
class MMSImage : public MMSWidget {
    private:
        string         className;
        MMSImageClass  *imageClass;
        MMSImageClass  myImageClass;

        MMSFBSurface    *image;
        MMSIM_DESC_SUF  *image_suf;
        unsigned int    image_curr_index;
        MMSFBSurface    *selimage;
        MMSIM_DESC_SUF  *selimage_suf;
        unsigned int    selimage_curr_index;

        MMSFBSurface    *image_p;
        MMSIM_DESC_SUF  *image_p_suf;
        unsigned int    image_p_curr_index;
        MMSFBSurface    *selimage_p;
        MMSIM_DESC_SUF  *selimage_p_suf;
        unsigned int    selimage_p_curr_index;
        
        MMSFBSurface    *image_i;
        MMSIM_DESC_SUF  *image_i_suf;
        unsigned int    image_i_curr_index;
        MMSFBSurface    *selimage_i;
        MMSIM_DESC_SUF  *selimage_i_suf;
        unsigned int    selimage_i_curr_index;

        bool    image_loaded;
        bool    image_p_loaded;
        bool    image_i_loaded;
        bool    selimage_loaded;
        bool    selimage_p_loaded;
        bool    selimage_i_loaded;

        class MMSImageThread  *imageThread;

        bool create(MMSWindow *root, string className, MMSTheme *theme);

        void loadMyImage(string path, string filename, MMSFBSurface **surface, MMSIM_DESC_SUF **surfdesc, unsigned int *index);

        void workWithRatio(MMSFBSurface *suf, DFBRectangle *surfaceGeom);
        
    public:
        MMSImage(MMSWindow *root, string className, MMSTheme *theme = NULL);
        ~MMSImage();
        MMSWIDGETTYPE getType() { return MMSWIDGETTYPE_IMAGE; }

        MMSWidget *copyWidget();

        bool init();
        bool draw(bool *backgroundFilled = NULL);

        void setVisible(bool visible, bool refresh = true);

    public:
        /* theme access methods */
        string getImagePath();
        string getImageName();
        string getSelImagePath();
        string getSelImageName();
        string getImagePath_p();
        string getImageName_p();
        string getSelImagePath_p();
        string getSelImageName_p();
        string getImagePath_i();
        string getImageName_i();
        string getSelImagePath_i();
        string getSelImageName_i();
        bool getUseRatio();
        bool getFitWidth();
        bool getFitHeight();
        MMSALIGNMENT getAlignment();

        void setImagePath(string imagepath, bool load = true, bool refresh = true);
        void setImageName(string imagename, bool load = true, bool refresh = true);
        void setImage(string imagepath, string imagename, bool load = true, bool refresh = true);
        void setSelImagePath(string selimagepath, bool load = true, bool refresh = true);
        void setSelImageName(string selimagename, bool load = true, bool refresh = true);
        void setSelImage(string selimagepath, string selimagename, bool load = true, bool refresh = true);
        void setImagePath_p(string imagepath_p, bool load = true, bool refresh = true);
        void setImageName_p(string imagename_p, bool load = true, bool refresh = true);
        void setImage_p(string imagepath_p, string imagename_p, bool load = true, bool refresh = true);
        void setSelImagePath_p(string selimagepath_p, bool load = true, bool refresh = true);
        void setSelImageName_p(string selimagename_p, bool load = true, bool refresh = true);
        void setSelImage_p(string selimagepath_p, string selimagename_p, bool load = true, bool refresh = true);
        void setImagePath_i(string imagepath_i, bool load = true, bool refresh = true);
        void setImageName_i(string imagename_i, bool load = true, bool refresh = true);
        void setImage_i(string imagepath_i, string imagename_i, bool load = true, bool refresh = true);
        void setSelImagePath_i(string selimagepath_i, bool load = true, bool refresh = true);
        void setSelImageName_i(string selimagename_i, bool load = true, bool refresh = true);
        void setSelImage_i(string selimagepath_i, string selimagename_i, bool load = true, bool refresh = true);
        void setUseRatio(bool useratio, bool refresh = true);
        void setFitWidth(bool fitwidth, bool refresh = true);
        void setFitHeight(bool fitheight, bool refresh = true);
        void setAlignment(MMSALIGNMENT alignment, bool refresh = true);

        void updateFromThemeClass(MMSImageClass *themeClass);

    friend class MMSImageThread;
};

#endif /*MMSIMAGE_H_*/
