/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mmsdata/mmsepgdao.h"

MMSEPGDAO::MMSEPGDAO(IMMSDB *connection) {
    setMMSDBConnection(connection);
}

void MMSEPGDAO::setMMSDBConnection(IMMSDB *connection) {
    this->dbConnection = connection;
}

IMMSDB *MMSEPGDAO::getMMSDBConnection() {
    return this->dbConnection;   
}

void MMSEPGDAO::insert(MMSEPGData data) {
    /* do the insert */
    data->setStartDT(data->getStartDate() + " " + data->getStartTime());
    data->setEndDT(data->getEndDate() + " " + data->getEndTime());
    try {
	    this->getMMSDBConnection()->query(
	        "insert into EPG(TVChannelID,StartDate,StartTime,StartDT,StartStamp, \
            EndDate,EndTime,EndDT,EndStamp,Title,Description,LongDescription, \
	        Language,Content,Scrambled) \
	        values('"
	        + iToStr((data->channel)?data->channel->getId():0) + "','"
	        + data->getStartDate() + "','"
            + data->getStartTime() + "','"
            + data->getStartDT() + "','"
            + iToStr(data->getStartStamp()) + "','"
	        + data->getEndDate() + "','"
            + data->getEndTime() + "','"
            + data->getEndDT() + "','"
            + iToStr(data->getEndStamp()) + "','"
	        + maskChars(data->getTitle()) + "','"
            + maskChars(data->getDescription()) + "','"
            + maskChars(data->getLongDescription()) + "','"
	    	+ maskChars(data->getLanguage()) + "','"
	    	+ maskChars(data->getContent()) + "','"
	    	+ (data->getScrambled() ? 't' : 'f') + "')");
    } catch(MMSError *error) {
		/* failed to insert, try update */
		update(data);
		return;
    }

    /* set the ID */
    data->setId(this->getMMSDBConnection()->getLastInsertedID());
}

void MMSEPGDAO::update(MMSEPGData data) {
    /* do the update */
    data->setStartDT(data->getStartDate() + " " + data->getStartTime());
    data->setEndDT(data->getEndDate() + " " + data->getEndTime());
    this->getMMSDBConnection()->query(
        "update EPG set TVChannelID='" + iToStr((data->channel)?data->channel->getId():0) + "',"
        + "StartDate='" + data->getStartDate() + "',"
        + "StartTime='" + data->getStartTime() + "',"
        + "StartDT='" + data->getStartDT() + "',"
        + "StartStamp='" + iToStr(data->getStartStamp()) + "',"
        + "EndDate='" + data->getEndDate() + "',"
        + "EndTime='" + data->getEndTime() + "',"
        + "EndDT='" + data->getEndDT() + "',"
        + "EndStamp='" + iToStr(data->getEndStamp()) + "',"
        + "Title='" + maskChars(data->getTitle()) + "',"
        + "Description='" + maskChars(data->getDescription()) + "',"
        + "LongDescription='" + maskChars(data->getLongDescription()) + "',"
        + "Language='" + maskChars(data->getLanguage()) + "',"
        + "Content='" + maskChars(data->getContent()) + "',"
        + "Scrambled='" + (data->getScrambled() ? 't' : 'f') + "' "
        "where ID = '" + iToStr(data->getId()) + "'");
}

void MMSEPGDAO::save(MMSEPGData data) {
    /* check if ID is set */
    if (data->getId()<0)
        /* no, have to insert */
        insert(data);
    else
        /* yes, have to update */
        update(data);
}

void MMSEPGDAO::save(vector<MMSEPGData> dataList) {
	this->getMMSDBConnection()->startTransaction();
    /* for each item */
    unsigned int lsize = dataList.size();
    for(unsigned i=0; i< lsize; i++) {
        /* check if ID is set */
        if (dataList.at(i)->getId()<0)
            /* no, have to insert */
            insert(dataList.at(i));
        else
            /* yes, have to update */
            update(dataList.at(i));
        if((i % 500) == 0) {
        	this->getMMSDBConnection()->commitTransaction();
        	this->getMMSDBConnection()->startTransaction();
            usleep(250000);
        }
    }
	this->getMMSDBConnection()->commitTransaction();
}

void MMSEPGDAO::removeOldData() {
    string          datestr, timestr;
    time_t          tm;

    /* remove entries that are older than 10 hours */
    tm = time(NULL) - 36000;
    getCurrentTimeString(NULL, &datestr, &timestr, &tm);
    string dt = datestr + " " + timestr;
   
    this->getMMSDBConnection()->query("delete from EPG where StartDT < \"" + dt + "\"");
}

_MMSEPGData* MMSEPGDAO::moveRecordToData(MMSRecordSet &rs) {
    _MMSEPGData *data = new _MMSEPGData;

    data->setId(atoi(rs["ID"].c_str()));
    data->setStartDate(rs["StartDate"]);
    data->setStartTime(rs["StartTime"]);
    data->setStartDT(rs["StartDT"]);
    data->setStartStamp(atoi(rs["StartStamp"].c_str()));
    data->setEndDate(rs["EndDate"]);
    data->setEndTime(rs["EndTime"]);
    data->setEndDT(rs["EndDT"]);
    data->setEndStamp(atoi(rs["EndStamp"].c_str()));
    data->setTitle(rs["Title"]);
    data->setDescription(rs["Description"]);
    data->setLongDescription(rs["LongDescription"]);
    
    return data;
}

MMSEPGData MMSEPGDAO::findCurrentEPGDataByChannel(MMSTVChannelData* channel) {
    _MMSEPGData     *data = NULL;
    MMSRecordSet    rs;
    time_t             now;
    
    time(&now);

    this->getMMSDBConnection()->query("select * from EPG where TVChannelID = " + iToStr(channel->getId())
                + " AND StartStamp <= '" + iToStr(now) + "' AND EndStamp >= '" + iToStr(now) + "' order by StartStamp",
                &rs);

    if(rs.getCount() > 0) {
    	data = moveRecordToData(rs);
    	data->setChannel(channel);
    }
    
    return MMSEPGData(data);
}

vector<MMSEPGData> MMSEPGDAO::findNextEPGDataByChannel(MMSTVChannelData* channel, const unsigned int limit) {
    vector<MMSEPGData> list;
    MMSRecordSet       rs;
    time_t             now;
    
    time(&now);

    string query = "select * from EPG where TVChannelID = " + iToStr(channel->getId()) +
                   " AND StartStamp > '" + iToStr(now) + "' order by StartStamp";
    if(limit > 0) query += " limit " + iToStr(limit);
    this->getMMSDBConnection()->query(query, &rs);

    if(rs.getCount() > 0) {
	    do {
	        _MMSEPGData *data = moveRecordToData(rs);
	        data->setChannel(channel);
	        list.push_back(MMSEPGData(data));
	    } while(rs.next() == true);
    }     

    return list;
}

/**
 * Combination of MMSEPGDAO::findCurrentEPGDataByChannel() and
 * MMSEPGDAO::findNextEPGDataByChannel().
 * 
 * It selects a given amount of EPG entries for a channel including
 * the current and the following shows.
 * 
 * @param	channel	[in]	channel to look for
 * @param   limit	[in]	maximum number of entries
 * 
 * @return	vector of EPG entries that matched
 *
 * @see		MMSEPGDAO::findCurrentEPGDataByChannel()
 * @see		MMSEPGDAO::findNextEPGDataByChannel()
 */
vector<MMSEPGData> MMSEPGDAO::findEPGDataByChannel(MMSTVChannelData* channel, const unsigned int limit) {
    vector<MMSEPGData> list;
    MMSRecordSet       rs;
    time_t             now;
    
    time(&now);

    string query = "select * from EPG where TVChannelID = " + iToStr(channel->getId()) +
                   " AND ((StartStamp <= '" + iToStr(now) + "' AND EndStamp >= '" + iToStr(now) + "') OR (EndStamp > '" + iToStr(now) +
                   "')) order by StartStamp";
    if(limit > 0) query += " limit " + iToStr(limit);
    this->getMMSDBConnection()->query(query, &rs);

    if(rs.getCount() > 0) {
	    do {
	        _MMSEPGData *data = moveRecordToData(rs);
	        data->setChannel(channel);
	        list.push_back(MMSEPGData(data));
	    } while(rs.next() == true);
    }     

    return list;
}
