/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mmsdata/mmsepgservice.h"
#include "mmstools/interfaces/immsdb.h"
#include "mmsdata/mmsepgdao.h"

MMSEPGService::MMSEPGService(DataSource *datasource) :
    dbconn(NULL) {
    MMSDBConnMgr connMgr(datasource); 
    if((this->dbconn = connMgr.getConnection()))
    	this->dbconn->connect();
}

MMSEPGService::~MMSEPGService() {
	if(this->dbconn) {
	    this->dbconn->disconnect();
	    delete this->dbconn;
	}
}

void MMSEPGService::setEPGData(MMSEPGData data) {
    MMSEPGDAO myEPGDAO(this->dbconn);
    myEPGDAO.save(data);
}

void MMSEPGService::setEPGData(vector<MMSEPGData> dataList) {
	MMSEPGDAO myEPGDAO(this->dbconn);
    myEPGDAO.save(dataList);
}

void MMSEPGService::removeOldData() {
	MMSEPGDAO myEPGDAO(this->dbconn);
    myEPGDAO.removeOldData();
}

MMSEPGData MMSEPGService::getCurrentEPGDataByChannel(MMSTVChannelData* channel) {
	MMSEPGDAO epgDAO(this->dbconn);
    return epgDAO.findCurrentEPGDataByChannel(channel);
}

vector<MMSEPGData> MMSEPGService::getNextEPGDataByChannel(MMSTVChannelData* channel, const unsigned int limit) {
	MMSEPGDAO epgDAO(this->dbconn);
    vector<MMSEPGData> epgList = epgDAO.findNextEPGDataByChannel(channel, limit);

    return epgList;
}

vector<MMSEPGData> MMSEPGService::getEPGDataByChannel(MMSTVChannelData* channel, const unsigned int limit) {
	MMSEPGDAO epgDAO(this->dbconn);
    vector<MMSEPGData> epgList = epgDAO.findEPGDataByChannel(channel, limit);

    return epgList;
}
