/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *      Matthias Hardt   <mattmax@morphine.tv>                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mmsdata/mmsmusicdao.h"

void MMSMusicDAO::setMMSDBConnection(IMMSDB *connection) {
    this->dbConnection = connection;
}

IMMSDB *MMSMusicDAO::getMMSDBConnection() {
    return this->dbConnection;   
}

unsigned int MMSMusicDAO::getArtistID(const string artist, const bool update) {
    MMSRecordSet rs;
    
    if(!artist.empty()) {
        // check if artist is already present
        try {
            this->getMMSDBConnection()->query("SELECT ID FROM MusicArtist WHERE Name = '" + maskChars(artist) + "'", &rs);
            if(!rs["ID"].empty())
                return strToInt(rs["ID"]);
        }
        catch(MMSError *e) {
        	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
            return 0;
        }

        if(update) {
            try {
                this->getMMSDBConnection()->query("INSERT INTO MusicArtist(Name) VALUES('" + maskChars(artist) + "')");
                this->getMMSDBConnection()->query("SELECT ID FROM MusicArtist WHERE Name = '" + maskChars(artist) + "'", &rs);
                return strToInt(rs["ID"]);
            }
            catch(MMSError *e) {
            	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
                return 0;
            }
        }
    }

    return 0;
}

unsigned int MMSMusicDAO::getAlbumID(const string album, const unsigned int trackCount, const string cover, const bool update) {
    MMSRecordSet rs;
    
    if(!album.empty()) {
        // check if album is already present
        try {
            this->getMMSDBConnection()->query("SELECT ID FROM MusicAlbum WHERE Name = '" + maskChars(album) + "'", &rs);
            if(!rs["ID"].empty())
                return strToInt(rs["ID"]);
        }
        catch(MMSError *e) {
        	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
            return 0;
        }

        if(update) {
            try {
                string query = "INSERT INTO MusicAlbum(Name";
                if(trackCount > 0)
                    query += ", TrackCount";
                if(!cover.empty())
                    query += ", Cover";
                query += ") VALUES('";
                query += maskChars(album);
                if(trackCount > 0)
                    query += "', '" + iToStr(trackCount);
                if(!cover.empty())
                    query += "', '" + maskChars(cover);
                query += "')";
                this->getMMSDBConnection()->query(query);
                this->getMMSDBConnection()->query("SELECT ID FROM MusicAlbum WHERE Name = '" + maskChars(album) + "'", &rs);
                return strToInt(rs["ID"]);
            }
            catch(MMSError *e) {
            	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
                return 0;
            }
        }
    }

    return 0;
}

unsigned int MMSMusicDAO::getFormatID(const string format, const bool update) {
    MMSRecordSet rs;
    
    if(!format.empty()) {
        // check if format is already present
        try {
            this->getMMSDBConnection()->query("SELECT ID FROM MusicFormat WHERE Name = '" + maskChars(format) + "'", &rs);
            if(!rs["ID"].empty())
                return strToInt(rs["ID"]);
        }
        catch(MMSError *e) {
        	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
            return 0;
        }

        if(update) {
            try {
                this->getMMSDBConnection()->query("INSERT INTO MusicFormat(Name) VALUES('" + maskChars(format) + "')");
                this->getMMSDBConnection()->query("SELECT ID FROM MusicFormat WHERE Name = '" + maskChars(format) + "'", &rs);
                return strToInt(rs["ID"]);
            }
            catch(MMSError *e) {
            	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
                return 0;
            }
        }
    }

    return 0;
}

unsigned int MMSMusicDAO::getGenreID(const string genre, const bool update) {
    MMSRecordSet rs;
    
    if(!genre.empty()) {
        // check if genre is already present
        try {
            this->getMMSDBConnection()->query("SELECT ID FROM MusicGenre WHERE Name = '" + maskChars(genre) + "'", &rs);
            if(!rs["ID"].empty())
                return strToInt(rs["ID"]);
        }
        catch(MMSError *e) {
        	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
            return 0;
        }

        if(update) {
            try {
                this->getMMSDBConnection()->query("INSERT INTO MusicGenre(Name) VALUES('" + maskChars(genre) + "')");
                this->getMMSDBConnection()->query("SELECT ID FROM MusicGenre WHERE Name = '" + maskChars(genre) + "'", &rs);
                return strToInt(rs["ID"]);
            }
            catch(MMSError *e) {
            	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
                return 0;
            }
        }
    }

    return 0;
}

unsigned int MMSMusicDAO::getYearID(const string year, const bool update) {
    MMSRecordSet rs;
    
    if(!year.empty()) {
        // check if year is already present
        try {
            this->getMMSDBConnection()->query("SELECT ID FROM MusicYear WHERE Year = '" + maskChars(year) + "'", &rs);
            if(!rs["ID"].empty())
                return strToInt(rs["ID"]);
        }
        catch(MMSError *e) {
        	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
            return 0;
        }

        if(update) {
            try {
                this->getMMSDBConnection()->query("INSERT INTO MusicYear(Year) VALUES('" + maskChars(year) + "')");
                this->getMMSDBConnection()->query("SELECT ID FROM MusicYear WHERE Year = '" + maskChars(year) + "'", &rs);
                return strToInt(rs["ID"]);
            }
            catch(MMSError *e) {
            	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
                return 0;
            }
        }
    }

    return 0;
}

/**
 * Inserts the music data into the corresponding
 * tables.
 * 
 * 
 */
void MMSMusicDAO::insert(MMSMusicData *data) {
    unsigned int idArtist, idAlbum, idFormat, idGenre, idYear;
    string query; 

    try {
        idArtist = this->getArtistID(data->getArtist());
        idAlbum  = this->getAlbumID(data->getAlbum(), data->getTrackCount(), data->getCover());
        idFormat = this->getFormatID(data->getFormat());
        idGenre  = this->getGenreID(data->getGenre());
        idYear   = this->getYearID(data->getYear());
        if(data->getTitle().empty()) data->setTitle(data->getFilename());
        
        query = "INSERT INTO MusicFile(Filename, Title, IDArtist, IDAlbum, IDFormat, IDGenre, IDYear, Length";
        if(!data->getComment().empty())
            query += ", Comment";
        if(data->getTrackNumber() > 0)
            query += ", TrackNr";
        if(data->getRating() > 0)
            query += ", Rating";
        query += ") Values('";
        query += maskChars(data->getFilename());
        query += "', '" + maskChars(data->getTitle());
        query += "', '" + iToStr(idArtist);
        query += "', '" + iToStr(idAlbum);
        query += "', '" + iToStr(idFormat);
        query += "', '" + iToStr(idGenre);
        query += "', '" + iToStr(idYear);
        query += "', '" + iToStr(data->getLength());
        if(!data->getComment().empty())
            query += "', '" + maskChars(data->getComment());
        if(data->getTrackNumber() > 0)
            query += "', '" + iToStr(data->getTrackNumber());
        if(data->getRating() > 0)
            query += "', '" + iToStr(data->getRating());
        query += "')";
        
        this->getMMSDBConnection()->query(query);
    } catch(MMSError *error) {
		if (data->getId()==-1)
			throw error;
        /* failed to insert, try update */
        update(data);
        return;
    }

    /* set the ID */
    data->setId(this->getMMSDBConnection()->getLastInsertedID());

}

void MMSMusicDAO::update(MMSMusicData *data) {
    unsigned int idArtist, idAlbum, idFormat, idGenre, idYear;
    string query; 

    idArtist = this->getArtistID(data->getArtist());
    idAlbum  = this->getAlbumID(data->getAlbum(), data->getTrackCount(), data->getCover());
    idFormat = this->getFormatID(data->getFormat());
    idGenre  = this->getGenreID(data->getGenre());
    idYear   = this->getYearID(data->getYear());
    if(data->getTitle().empty()) data->setTitle(data->getFilename());

    query = "UPDATE MusicFile ";
    query += "SET Filename = '" + maskChars(data->getFilename());
    query += "', Title = '" + maskChars(data->getTitle());
    query += "', IDArtist = '" + iToStr(idArtist);
    query += "', IDAlbum = '" + iToStr(idAlbum);
    query += "', IDFormat = '" + iToStr(idFormat);
    query += "', IDGenre = '" + iToStr(idGenre);
    query += "', IDYear = '" + iToStr(idYear);
    query += "', Length = '" + iToStr(data->getLength());
    if(!data->getComment().empty())
        query += "', Comment = '" + maskChars(data->getComment());
    if(data->getTrackNumber() > 0)
        query += "', TrackNr = '" + iToStr(data->getTrackNumber());
    if(data->getRating() > 0)
        query += "', Rating = '" + iToStr(data->getRating());
    query += "' WHERE ID = '" + iToStr(data->getId()) + "'";
    
    this->getMMSDBConnection()->query(query);
}

MMSMusicData MMSMusicDAO::moveRecordToData(MMSRecordSet &rs) {
    MMSMusicData data;

    data.setId(atoi(rs["ID"].c_str()));
    data.setFilename(rs["Filename"]);
    data.setTitle(rs["Title"]);
    data.setArtist(rs["Artist"]);
    data.setAlbum(rs["Album"]);
    data.setGenre(rs["Genre"]);
    data.setYear(rs["Year"]);
    data.setComment(rs["Comment"]);
    data.setFormat(rs["Format"]);
    data.setTrackNumber(atoi(rs["TrackNr"].c_str()));
    data.setTrackCount(atoi(rs["TrackCount"].c_str()));
    data.setRating(atoi(rs["Rating"].c_str()));
    data.setLength(atoi(rs["Length"].c_str()));
    data.setCover(rs["Cover"]);
    
    return data;
}

MMSMusicDAO::MMSMusicDAO(IMMSDB *connection) {
    setMMSDBConnection(connection);
}

void MMSMusicDAO::save(MMSMusicData *data) {
    /* check if ID is set */
    if (data->getId()<0)
        /* no, have to insert */
        insert(data);
    else
        /* yes, have to update */
        update(data);

}

void MMSMusicDAO::save(vector<MMSMusicData *> dataList) {
    /* for each item */
    for(int i = dataList.size() - 1; i >= 0; --i) {
        /* check if ID is set */
        if(dataList.at(i)->getId()<0)
            /* no, have to insert */
            insert(dataList.at(i));
        else
            /* yes, have to update */
            update(dataList.at(i));
    }
}

void MMSMusicDAO::getStatistics(unsigned int &numSongs, unsigned int &numArtists, unsigned int &numAlbums, unsigned int &numGenres) {
    MMSRecordSet rs; 

    numSongs   = 0;
    numArtists = 0;
    numAlbums  = 0;
    numGenres  = 0;

    this->getMMSDBConnection()->query("SELECT SongCount, ArtistCount, AlbumCount, GenreCount FROM MusicStatistics WHERE ID = '0'", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return;

    /* set the values */
    numSongs   = atoi(rs["SongCount"].c_str());
    numArtists = atoi(rs["ArtistCount"].c_str());
    numAlbums  = atoi(rs["AlbumCount"].c_str());
    numGenres  = atoi(rs["GenreCount"].c_str());
}

void MMSMusicDAO::getStatistics(unsigned int &numSongs, unsigned int &numArtists, unsigned int &numAlbums, unsigned int &numGenres, string &lastPlayed) {
    MMSRecordSet rs; 

    numSongs   = 0;
    numArtists = 0;
    numAlbums  = 0;
    numGenres  = 0;
    lastPlayed = "";

    this->getMMSDBConnection()->query("SELECT SongCount, ArtistCount, AlbumCount, GenreCount, LastPlayed FROM MusicStatistics WHERE ID = '0'", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return;

    /* set the values */
    numSongs   = atoi(rs["SongCount"].c_str());
    numArtists = atoi(rs["ArtistCount"].c_str());
    numAlbums  = atoi(rs["AlbumCount"].c_str());
    numGenres  = atoi(rs["GenreCount"].c_str());
    lastPlayed = rs["LastPlayed"];
}

void MMSMusicDAO::setLastPlayed(const string lastPlayed) {
    string query = "UPDATE MusicStatistics ";
    query += "SET LastPlayed = '" + maskChars(lastPlayed);
    query += "' WHERE ID = '0'";
    
    this->getMMSDBConnection()->query(query);
}

vector <MMSMusicData *> MMSMusicDAO::getSongs() {
    vector <MMSMusicData *> ret;
    MMSRecordSet    rs;

    string query = "SELECT Filename, \
                           TrackNr, \
                           Title, \
                           Comment, \
                           Rating, \
                           Length, \
                           MusicArtist.Name AS Artist, \
                           MusicAlbum.Name AS Album, \
                           MusicAlbum.TrackCount AS TrackCount, \
                           MusicAlbum.Cover AS Cover, \
                           MusicFormat.Name AS Format, \
                           MusicGenre.Name AS Genre, \
                           MusicYear.Year AS Year \
                    FROM   MusicFile, \
                           MusicArtist, \
                           MusicAlbum, \
                           MusicFormat, \
                           MusicGenre, \
                           MusicYear \
                    WHERE  MusicFile.IDArtist = MusicArtist.ID \
                    AND    MusicFile.IDAlbum  = MusicAlbum.ID \
                    AND    MusicFile.IDFormat = MusicFormat.ID \
                    AND    MusicFile.IDGenre  = MusicGenre.ID \
                    AND    MusicFile.IDYear   = MusicYear.ID \
                    ORDER BY Album, TrackNr, Artist";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        MMSMusicData *data = new MMSMusicData;
        *data = moveRecordToData(rs);

        /* push to list */
        ret.push_back(data);
    }
    while(rs.next() == true);
     
    return ret;
}

vector <MMSMusicData *> MMSMusicDAO::getSongsByArtist(string artist) {
    vector <MMSMusicData *> ret;
    MMSRecordSet    rs; 

    string query = "SELECT Filename, \
                           TrackNr, \
                           Title, \
                           Comment, \
                           Rating, \
                           Length, \
                           MusicArtist.Name AS Artist, \
                           MusicAlbum.Name AS Album, \
                           MusicAlbum.TrackCount AS TrackCount, \
                           MusicAlbum.Cover AS Cover, \
                           MusicFormat.Name AS Format, \
                           MusicGenre.Name AS Genre, \
                           MusicYear.Year AS Year \
                    FROM   MusicFile, \
                           MusicArtist, \
                           MusicAlbum, \
                           MusicFormat, \
                           MusicGenre, \
                           MusicYear \
                    WHERE  MusicArtist.Name = '" + maskChars(artist) + "' \
                    AND    MusicFile.IDArtist = MusicArtist.ID \
                    AND    MusicFile.IDAlbum  = MusicAlbum.ID \
                    AND    MusicFile.IDFormat = MusicFormat.ID \
                    AND    MusicFile.IDGenre  = MusicGenre.ID \
                    AND    MusicFile.IDYear   = MusicYear.ID \
                    ORDER BY Album, TrackNr, Title";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        MMSMusicData *data = new MMSMusicData;
        *data = moveRecordToData(rs);

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

vector <MMSMusicData *> MMSMusicDAO::getSongsByAlbum(string album) {
    vector <MMSMusicData *> ret;
    MMSRecordSet    rs; 

    string query = "SELECT Filename, \
                           TrackNr, \
                           Title, \
                           Comment, \
                           Rating, \
                           Length, \
                           MusicArtist.Name AS Artist, \
                           MusicAlbum.Name AS Album, \
                           MusicAlbum.TrackCount AS TrackCount, \
                           MusicAlbum.Cover AS Cover, \
                           MusicFormat.Name AS Format, \
                           MusicGenre.Name AS Genre, \
                           MusicYear.Year AS Year \
                    FROM   MusicFile, \
                           MusicArtist, \
                           MusicAlbum, \
                           MusicFormat, \
                           MusicGenre, \
                           MusicYear \
                    WHERE  MusicAlbum.Name = '" + maskChars(album) + "' \
                    AND    MusicFile.IDArtist = MusicArtist.ID \
                    AND    MusicFile.IDAlbum  = MusicAlbum.ID \
                    AND    MusicFile.IDFormat = MusicFormat.ID \
                    AND    MusicFile.IDGenre  = MusicGenre.ID \
                    AND    MusicFile.IDYear   = MusicYear.ID \
                    ORDER BY TrackNr, Title";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        MMSMusicData *data = new MMSMusicData;
        *data = moveRecordToData(rs);

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

vector <MMSMusicData *> MMSMusicDAO::getSongsByGenre(string genre) {
    vector <MMSMusicData *> ret;
    MMSRecordSet    rs; 

    string query = "SELECT Filename, \
                           TrackNr, \
                           Title, \
                           Comment, \
                           Rating, \
                           Length, \
                           MusicArtist.Name AS Artist, \
                           MusicAlbum.Name AS Album, \
                           MusicAlbum.TrackCount AS TrackCount, \
                           MusicAlbum.Cover AS Cover, \
                           MusicFormat.Name AS Format, \
                           MusicGenre.Name AS Genre, \
                           MusicYear.Year AS Year \
                    FROM   MusicFile, \
                           MusicArtist, \
                           MusicAlbum, \
                           MusicFormat, \
                           MusicGenre, \
                           MusicYear \
                    WHERE  MusicGenre.Name = '" + maskChars(genre) + "' \
                    AND    MusicFile.IDArtist = MusicArtist.ID \
                    AND    MusicFile.IDAlbum  = MusicAlbum.ID \
                    AND    MusicFile.IDFormat = MusicFormat.ID \
                    AND    MusicFile.IDGenre  = MusicGenre.ID \
                    AND    MusicFile.IDYear   = MusicYear.ID \
                    ORDER BY Album, TrackNr, Artist, Title";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        MMSMusicData *data = new MMSMusicData;
        *data = moveRecordToData(rs);

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

vector <string *> MMSMusicDAO::getArtists() {
    vector <string *> ret;
    MMSRecordSet    rs; 

    this->getMMSDBConnection()->query("SELECT Name FROM MusicArtist ORDER BY Name", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        string *data = new string;
        *data = rs["Name"];
        trim(*data);
        if(data->empty()) {
            delete data;
            continue;
        }

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

vector <string *> MMSMusicDAO::getAlbums() {
    vector <string *> ret;
    MMSRecordSet    rs; 

    this->getMMSDBConnection()->query("SELECT Name FROM MusicAlbum GROUP BY Name", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        string *data = new string;
        *data = rs["Name"];
        trim(*data);
        if(data->empty()) {
            delete data;
            continue;
        }

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

vector <string *> MMSMusicDAO::getGenres() {
    vector <string *> ret;
    MMSRecordSet    rs; 

    this->getMMSDBConnection()->query("SELECT Name FROM MusicGenre GROUP BY Name", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        string *data = new string;
        *data = rs["Name"];
        trim(*data);
        if(data->empty()) {
            delete data;
            continue;
        }

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

MMSMusicData *MMSMusicDAO::getSongByFilename(string filename) {
    MMSMusicData *ret = NULL;
    MMSRecordSet rs; 
    
    string query = "SELECT Filename, \
                           TrackNr, \
                           Title, \
                           Comment, \
                           Rating, \
                           Length, \
                           MusicArtist.Name AS Artist, \
                           MusicAlbum.Name AS Album, \
                           MusicAlbum.TrackCount AS TrackCount, \
                           MusicAlbum.Cover AS Cover, \
                           MusicFormat.Name AS Format, \
                           MusicGenre.Name AS Genre, \
                           MusicYear.Year AS Year \
                    FROM   MusicFile, \
                           MusicArtist, \
                           MusicAlbum, \
                           MusicFormat, \
                           MusicGenre, \
                           MusicYear \
                    WHERE  Filename = '" + maskChars(filename) + "' \
                    AND    MusicFile.IDArtist = MusicArtist.ID \
                    AND    MusicFile.IDAlbum  = MusicAlbum.ID \
                    AND    MusicFile.IDFormat = MusicFormat.ID \
                    AND    MusicFile.IDGenre  = MusicGenre.ID \
                    AND    MusicFile.IDYear   = MusicYear.ID";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;
    ret = new MMSMusicData();
    
    *ret = moveRecordToData(rs);
    
    return ret;
}
