/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *      Matthias Hardt   <mattmax@morphine.tv>                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mmsdata/mmsmusicservice.h"

MMSMusicService::MMSMusicService(DataSource *datasource) :
    dbconn(NULL) {
    MMSDBConnMgr connMgr(datasource); 
    if((this->dbconn = connMgr.getConnection()))
    	this->dbconn->connect();
}

MMSMusicService::~MMSMusicService() {
	if(this->dbconn) {
		this->dbconn->disconnect();
		delete this->dbconn;
	}
}

void MMSMusicService::setMusicData(MMSMusicData *data) {
    MMSMusicDAO dao(this->dbconn);
    dao.save(data);
}

void MMSMusicService::setMusicData(vector<MMSMusicData *> dataList) {
    MMSMusicDAO dao(this->dbconn);
    dao.save(dataList);
}

void MMSMusicService::getStatistics(unsigned int &numSongs, unsigned int &numArtists, unsigned int &numAlbums, unsigned int &numGenres) {
    MMSMusicDAO dao(this->dbconn);
    dao.getStatistics(numSongs, numArtists, numAlbums, numGenres);
}

void MMSMusicService::getStatistics(unsigned int &numSongs, unsigned int &numArtists, unsigned int &numAlbums, unsigned int &numGenres, string &lastPlayed) {
    MMSMusicDAO dao(this->dbconn);
    dao.getStatistics(numSongs, numArtists, numAlbums, numGenres, lastPlayed);
}

void MMSMusicService::setLastPlayed(const string lastPlayed) {
    MMSMusicDAO dao(this->dbconn);
    dao.setLastPlayed(lastPlayed);
}

vector <string *> MMSMusicService::getAlbums() {
    MMSMusicDAO dao(this->dbconn);
    return dao.getAlbums();
}

vector <string *> MMSMusicService::getArtists() {
    MMSMusicDAO dao(this->dbconn);
    return dao.getArtists();
}

vector <string *> MMSMusicService::getGenres() {
    MMSMusicDAO dao(this->dbconn);
    return dao.getGenres();
}

MMSMusicData *MMSMusicService::getSongByFilename(string filename) {
    MMSMusicDAO dao(this->dbconn);
    return dao.getSongByFilename(filename);
}

vector <MMSMusicData *> MMSMusicService::getSongs() {
    MMSMusicDAO dao(this->dbconn);
    return dao.getSongs();
}

vector <MMSMusicData *> MMSMusicService::getSongsByAlbum(string album) {
    MMSMusicDAO dao(this->dbconn);
    return dao.getSongsByAlbum(album);
}

vector <MMSMusicData *> MMSMusicService::getSongsByArtist(string artist) {
    MMSMusicDAO dao(this->dbconn);
    return dao.getSongsByArtist(artist);
}

vector <MMSMusicData *> MMSMusicService::getSongsByGenre(string genre) {
    MMSMusicDAO dao(this->dbconn);
    return dao.getSongsByGenre(genre);
}
