/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mmsdata/mmsnewsdao.h"

MMSNewsDAO::MMSNewsDAO(IMMSDB *connection) {
    setMMSDBConnection(connection);
}

void MMSNewsDAO::setMMSDBConnection(IMMSDB *connection) {
    this->dbConnection = connection;
}

IMMSDB *MMSNewsDAO::getMMSDBConnection() {
    return this->dbConnection;   
}

void MMSNewsDAO::save(MMSNewsData *data) {
    /* do the insert */
    this->getMMSDBConnection()->query(
        "insert into News(ImportSourceID,Title,Link,Description, \
        CreateDate,CreateTime,CreateTimeStamp,DescText,DescImages, \
        PubDate,Author,Category,Guid) values('"
        + iToStr(data->getSourceId()) + "','"
        + maskChars(data->getTitle()) + "','"
        + data->getLink() + "','"
        + maskChars(data->getDescription()) + "','"
        + data->getCreateDate() + "','"
        + data->getCreateTime() + "','"
        + iToStr(data->getCreateTimeStamp()) + "','"
        + maskChars(data->getDescText()) + "','"
        + data->getDescImages() + "','"
        + data->getPubDate() + "','"
        + maskChars(data->getAuthor()) + "','"
        + maskChars(data->getCategory()) + "','"
        + data->getGuid() + "')");

    /* set the ID */
    data->setId(this->getMMSDBConnection()->getLastInsertedID());
}

void MMSNewsDAO::update(MMSNewsData *data) {
    /* do the update */
    this->getMMSDBConnection()->query(
        "update News set Title='" + maskChars(data->getTitle()) + "',"
        + "Link='" + data->getLink() + "',"
        + "Description='" + maskChars(data->getDescription()) + "',"
        + "CreateDate='" + data->getCreateDate() + "',"
        + "CreateTime='" + data->getCreateTime() + "',"
        + "CreateTimeStamp='" + iToStr(data->getCreateTimeStamp()) + "',"
        + "DescText='" + maskChars(data->getDescText()) + "',"
        + "DescImages='" + data->getDescImages() + "',"
        + "PubDate='" + data->getPubDate() + "',"
        + "Author='" + maskChars(data->getAuthor()) + "',"
        + "Category='" + maskChars(data->getCategory()) + "',"
        + "Guid='" + data->getGuid() + "' "
        "where ID = '" + iToStr(data->getId()) + "'");
}

void MMSNewsDAO::saveOrUpdate(MMSNewsData *data) {
    /* check if ID is set */
    if (data->getId()<0)
        /* no, have to save */
        save(data);
    else
        /* yes, have to update */
        update(data);
}

void MMSNewsDAO::saveOrUpdate(vector<MMSNewsData *> dataList) {
    /* for each item */
    for (unsigned i=0;i<dataList.size();i++) {
        /* check if ID is set */
        if (dataList.at(i)->getId()<0)
            /* no, have to save */
            save(dataList.at(i));
        else
            /* yes, have to update */
            update(dataList.at(i));
    }
}

MMSNewsData MMSNewsDAO::moveRecordToData(MMSRecordSet &rs) {
    MMSNewsData data;

    data.setId(atoi(rs["ID"].c_str()));
    data.setSourceId(atoi(rs["ImportSourceID"].c_str()));
    data.setTitle(rs["Title"]);
    data.setLink(rs["Link"]);
    data.setDescription(rs["Description"]);
    data.setCreateDate(rs["CreateDate"]);
    data.setCreateTime(rs["CreateTime"]);
    data.setCreateTimeStamp(atoi(rs["CreateTimeStamp"].c_str()));
    data.setDescText(rs["DescText"]);
    data.setDescImages(rs["DescImages"]);
    data.setPubDate(rs["PubDate"]);
    data.setAuthor(rs["Author"]);
    data.setCategory(rs["Category"]);
    data.setGuid(rs["Guid"]);

    return data;
}

vector<MMSNewsData *> MMSNewsDAO::findAllNews() {
    vector<MMSNewsData *>   list;
    MMSRecordSet            rs;

    /* do query */
    this->getMMSDBConnection()->query("select * from News", &rs);

    /* check if result is empty */
    if (rs.getCount()==0) return list;

    /* for each result record */
    do {
        /* set the values */
        MMSNewsData *data = new MMSNewsData;
        *data = moveRecordToData(rs);

        /* push to list */
        list.push_back(data);
    } while(rs.next() == true);
        
    return list;
}

vector<MMSNewsData *> MMSNewsDAO::findAllNewsBySourceID(int sourceId) {
    vector<MMSNewsData *>   list;
    MMSRecordSet            rs;

    /* do query */
    this->getMMSDBConnection()->query("select * from News where ImportSourceID = "
                                    + iToStr(sourceId) + " order by pubDate", &rs);

    /* check if result is empty */
    if (rs.getCount()==0) return list;

    /* for each result record */
    do {
        /* set the values */
        MMSNewsData *data = new MMSNewsData;
        *data = moveRecordToData(rs);

        /* push to list */
        list.push_back(data);
    } while(rs.next() == true);
        
    return list;
}

MMSNewsData *MMSNewsDAO::findNewsByID(int id) {
    MMSRecordSet    rs;
    MMSNewsData     *data;

    this->getMMSDBConnection()->query("select * from News where ID = '"
                                    + iToStr(id) + "'", &rs);

    /* check if result is empty */
    if (rs.getCount()==0)
        return NULL;

    /* return values */
    data = new MMSNewsData;
    *data = moveRecordToData(rs);
    return data;
}

MMSNewsData *MMSNewsDAO::findNewsByNews(MMSNewsData news) {
    MMSRecordSet    rs;
    MMSNewsData     *data;

    if (news.getId()>=0)
        /* id is given, check with it */
        this->getMMSDBConnection()->query("select * from News where ID = '"
                                        + iToStr(news.getId()) + "'", &rs);
    else
    if (news.getGuid()!="")
        /* unique id is given, check with it */
        this->getMMSDBConnection()->query("select * from News where ImportSourceID = '"
                            + iToStr(news.getSourceId()) + "' and Guid = '"
                            + news.getGuid() + "'", &rs);
    else
    if (news.getPubDate()!="")
        /* pubDate is given, check with it and title */
        this->getMMSDBConnection()->query("select * from News where ImportSourceID = '"
                            + iToStr(news.getSourceId()) + "' and PubDate = '"
                            + news.getPubDate() + "' and Title = '"
                            + maskChars(news.getTitle()) + "'", &rs);
    else
        /* check only with title */
        this->getMMSDBConnection()->query("select * from News where ImportSourceID = '"
                            + iToStr(news.getSourceId()) + "' and Title = '"
                            + maskChars(news.getTitle()) + "'", &rs);

    /* check if result is empty */
    if (rs.getCount()==0)
        return NULL;

    /* return values */
    data = new MMSNewsData;
    *data = moveRecordToData(rs);
    return data;
}
