/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mmsdata/mmsnewsinfodao.h"

MMSNewsInfoDAO::MMSNewsInfoDAO(IMMSDB *connection) {
    setMMSDBConnection(connection);
}

void MMSNewsInfoDAO::setMMSDBConnection(IMMSDB *connection) {
    this->dbConnection = connection;
}

IMMSDB *MMSNewsInfoDAO::getMMSDBConnection() {
    return this->dbConnection;   
}

void MMSNewsInfoDAO::save(MMSNewsInfoData *data) {
    /* do the insert */
    this->getMMSDBConnection()->query(
        "insert into NewsInfo(ImportSourceID,Name,Title,Link,Description, \
        LastDate,LastTime,LastTimeStamp,LastRC,LastMSG,PubDate,Copyright) \
        values('"
        + iToStr(data->getSourceId()) + "','"
        + maskChars(data->getName()) + "','"
        + maskChars(data->getTitle()) + "','"
        + data->getLink() + "','"
        + maskChars(data->getDescription()) + "','"
        + data->getLastDate() + "','"
        + data->getLastTime() + "','"
        + iToStr(data->getLastTimeStamp()) + "','"
        + iToStr(data->getLastRC()) + "','"
        + data->getLastMSG() + "','"
        + data->getPubDate() + "','"
        + maskChars(data->getCopyright()) + "')");

    /* set the ID */
    data->setId(this->getMMSDBConnection()->getLastInsertedID());
}

void MMSNewsInfoDAO::update(MMSNewsInfoData *data) {
    /* do the update */
    this->getMMSDBConnection()->query(
        "update NewsInfo set Name='" + maskChars(data->getName()) + "',"
        + "Title='" + maskChars(data->getTitle()) + "',"
        + "Link='" + data->getLink() + "',"
        + "Description='" + maskChars(data->getDescription()) + "',"
        + "LastDate='" + data->getLastDate() + "',"
        + "LastTime='" + data->getLastTime() + "',"
        + "LastTimeStamp='" + iToStr(data->getLastTimeStamp()) + "',"
        + "LastRC='" + iToStr(data->getLastRC()) + "',"
        + "LastMSG='" + data->getLastMSG() + "',"
        + "PubDate='" + data->getPubDate() + "',"
        + "Copyright='" + maskChars(data->getCopyright()) + "' "
        "where ID = '" + iToStr(data->getId()) + "'");
}

void MMSNewsInfoDAO::saveOrUpdate(MMSNewsInfoData *data) {
    /* check if ID is set */
    if (data->getId()<0)
        /* no, have to save */
        save(data);
    else
        /* yes, have to update */
        update(data);
}

void MMSNewsInfoDAO::saveOrUpdate(vector<MMSNewsInfoData *> dataList) {
    /* for each item */
    for (unsigned i=0;i<dataList.size();i++) {
        /* check if ID is set */
        if (dataList.at(i)->getId()<0)
            /* no, have to save */
            save(dataList.at(i));
        else
            /* yes, have to update */
            update(dataList.at(i));
    }
}

MMSNewsInfoData MMSNewsInfoDAO::moveRecordToData(MMSRecordSet &rs) {
    MMSNewsInfoData data;

    data.setId(atoi(rs["ID"].c_str()));
    data.setSourceId(atoi(rs["ImportSourceID"].c_str()));
    data.setName(rs["Name"]);
    data.setTitle(rs["Title"]);
    data.setLink(rs["Link"]);
    data.setDescription(rs["Description"]);
    data.setLastDate(rs["LastDate"]);
    data.setLastTime(rs["LastTime"]);
    data.setLastTimeStamp(atoi(rs["LastTimeStamp"].c_str()));
    data.setLastRC(atoi(rs["LastRC"].c_str()));
    data.setLastMSG(rs["LastMSG"]);
    data.setPubDate(rs["PubDate"]);
    data.setCopyright(rs["Copyright"]);
    
    return data;
}

MMSNewsInfoData *MMSNewsInfoDAO::findNewsInfoBySourceID(int sourceId) {
    MMSNewsInfoData *data;
    MMSRecordSet    rs;

    /* do query */
    this->getMMSDBConnection()->query("select * from NewsInfo where ImportSourceID = "
                                    + iToStr(sourceId), &rs);

    /* check if result is empty */
    if (rs.getCount()==0)
        return NULL;

    /* fill the return data */
    data = new MMSNewsInfoData;
    *data = moveRecordToData(rs);
    return data;
}

vector<MMSNewsInfoData *> MMSNewsInfoDAO::findAllNewsInfos() {
    vector<MMSNewsInfoData *>   list;
    MMSRecordSet                rs;

    /* do query */
    this->getMMSDBConnection()->query("select * from NewsInfo", &rs);

    /* check if result is empty */
    if (rs.getCount()==0) return list;

    /* for each result record */
    do {
        /* set the values */
        MMSNewsInfoData *data = new MMSNewsInfoData;
        *data = moveRecordToData(rs);

        /* push to list */
        list.push_back(data);
    } while(rs.next() == true);
        
    return list;
}
