/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *      Matthias Hardt   <mattmax@morphine.tv>                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mmsdata/mmsphotodao.h"

void MMSPhotoDAO::setMMSDBConnection(IMMSDB *connection) {
    this->dbConnection = connection;
}

IMMSDB *MMSPhotoDAO::getMMSDBConnection() {
    return this->dbConnection;   
}

unsigned int MMSPhotoDAO::getPlaceID(const string place, const bool update) {
    MMSRecordSet rs;
    
    if(!place.empty()) {
        // check if place is already present
        try {
            this->getMMSDBConnection()->query("SELECT ID FROM PhotoPlace WHERE Name = '" + maskChars(place) + "'", &rs);
            if(!rs["ID"].empty())
                return strToInt(rs["ID"]);
        }
        catch(MMSError *e) {
        	DEBUGMSG("MMSPhotoDao", "Error: %s", e->getMessage().c_str());
            return 0;
        }

        if(update) {
            try {
                string query = "INSERT INTO PhotoPlace(Name) VALUES('";
                query += maskChars(place);
                query += "')";
                this->getMMSDBConnection()->query(query);
                this->getMMSDBConnection()->query("SELECT ID FROM PhotoPlace WHERE Name = '" + maskChars(place) + "'", &rs);
                return strToInt(rs["ID"]);
            }
            catch(MMSError *e) {
            	DEBUGMSG("MMSPhotoDao", "Error: %s", e->getMessage().c_str());
                return 0;
            }
        }
    }

    return 0;
}

unsigned int MMSPhotoDAO::getCameraID(const string camera, const bool update) {
    MMSRecordSet rs;
    
    if(!camera.empty()) {
        // check if place is already present
        try {
            this->getMMSDBConnection()->query("SELECT ID FROM PhotoCamera WHERE Name = '" + maskChars(camera) + "'", &rs);
            if(!rs["ID"].empty())
                return strToInt(rs["ID"]);
        }
        catch(MMSError *e) {
        	DEBUGMSG("MMSPhotoDao", "Error: %s", e->getMessage().c_str());
            return 0;
        }

        if(update) {
            try {
                string query = "INSERT INTO PhotoCamera(Name) VALUES('";
                query += maskChars(camera);
                query += "')";
                this->getMMSDBConnection()->query(query);
                this->getMMSDBConnection()->query("SELECT ID FROM PhotoCamera WHERE Name = '" + maskChars(camera) + "'", &rs);
                return strToInt(rs["ID"]);
            }
            catch(MMSError *e) {
            	DEBUGMSG("MMSPhotoDao", "Error: %s", e->getMessage().c_str());
                return 0;
            }
        }
    }

    return 0;
}

unsigned int MMSPhotoDAO::getAlbumID(const string album, const string albumCover, const bool update) {
    MMSRecordSet rs;
    
    if(!album.empty()) {
        // check if album is already present
        try {
            this->getMMSDBConnection()->query("SELECT ID FROM PhotoAlbum WHERE Name = '" + maskChars(album) + "'", &rs);
            if(!rs["ID"].empty())
                return strToInt(rs["ID"]);
        }
        catch(MMSError *e) {
        	DEBUGMSG("MMSPhotoDao", "Error: %s", e->getMessage().c_str());
            return 0;
        }

        if(update) {
            try {
                string query = "INSERT INTO PhotoAlbum(Name";
                if(!albumCover.empty())
                    query += ", Cover";
                query += ") VALUES('";
                query += maskChars(album);
                if(!albumCover.empty())
                    query += "', '" + maskChars(albumCover);
                query += "')";
                this->getMMSDBConnection()->query(query);
                this->getMMSDBConnection()->query("SELECT ID FROM PhotoAlbum WHERE Name = '" + maskChars(album) + "'", &rs);
                return strToInt(rs["ID"]);
            }
            catch(MMSError *e) {
            	DEBUGMSG("MMSPhotoDao", "Error: %s", e->getMessage().c_str());
                return 0;
            }
        }
    }

    return 0;
}

unsigned int MMSPhotoDAO::getFormatID(const string format, const bool update) {
    MMSRecordSet rs;
    
    if(!format.empty()) {
        // check if format is already present
        try {
            this->getMMSDBConnection()->query("SELECT ID FROM PhotoFormat WHERE Name = '" + maskChars(format) + "'", &rs);
            if(!rs["ID"].empty())
                return strToInt(rs["ID"]);
        }
        catch(MMSError *e) {
        	DEBUGMSG("MMSPhotoDao", "Error: %s", e->getMessage().c_str());
            return 0;
        }

        if(update) {
            try {
                this->getMMSDBConnection()->query("INSERT INTO PhotoFormat(Name) VALUES('" + maskChars(format) + "')");
                this->getMMSDBConnection()->query("SELECT ID FROM PhotoFormat WHERE Name = '" + maskChars(format) + "'", &rs);
                return strToInt(rs["ID"]);
            }
            catch(MMSError *e) {
            	DEBUGMSG("MMSPhotoDao", "Error: %s", e->getMessage().c_str());
                return 0;
            }
        }
    }

    return 0;
}

/**
 * Inserts the photo data into the corresponding
 * tables.
 * 
 * 
 */
void MMSPhotoDAO::insert(MMSPhotoData *data) {
    unsigned int idPlace, idCamera, idAlbum, idFormat;
    string query; 

    try {
        idPlace  = this->getPlaceID(data->getPlace());
        idCamera = this->getCameraID(data->getCamera());
        idAlbum  = this->getAlbumID(data->getAlbum(), data->getAlbumCover());
        idFormat = this->getFormatID(data->getFormat());
        if(data->getTitle().empty()) data->setTitle(data->getFilename());
        
        query = "INSERT INTO PhotoFile(Filename, Title, Date, IDPlace, IDCamera, IDAlbum, IDFormat";
        if(!data->getComment().empty())
            query += ", Comment";
        if(data->getRating() > 0)
            query += ", Rating";
        query += ") VALUES('";
        query += maskChars(data->getFilename());
        query += "', '" + maskChars(data->getTitle());
        query += "', '" + maskChars(data->getDate());
        query += "', '" + iToStr(idPlace);
        query += "', '" + iToStr(idCamera);
        query += "', '" + iToStr(idAlbum);
        query += "', '" + iToStr(idFormat);
        if(!data->getComment().empty())
            query += "', '" + maskChars(data->getComment());
        if(data->getRating() > 0)
            query += "', '" + iToStr(data->getRating());
        query += "')";
        
        this->getMMSDBConnection()->query(query);
    } catch(MMSError *error) {
        if (data->getId()==-1)
            throw error;
        /* failed to insert, try update */
        update(data);
        return;
    }

    /* set the ID */
    data->setId(this->getMMSDBConnection()->getLastInsertedID());

}

void MMSPhotoDAO::update(MMSPhotoData *data) {
    unsigned int idPlace, idCamera, idAlbum, idFormat;
    string query; 

    idPlace  = this->getPlaceID(data->getPlace());
    idCamera = this->getCameraID(data->getCamera());
        idAlbum  = this->getAlbumID(data->getAlbum(), data->getAlbumCover());
    idFormat = this->getFormatID(data->getFormat());

    query = "UPDATE PhotoFile ";
    query += "SET Filename = '" + maskChars(data->getFilename());
    query += "', Title = '" + maskChars(data->getTitle());
    query += "', Date = '" + maskChars(data->getDate());
    query += "', IDPlace = '" + iToStr(idPlace);
    query += "', IDCamera = '" + iToStr(idCamera);
    query += "', IDAlbum = '" + iToStr(idAlbum);
    query += "', IDFormat = '" + iToStr(idFormat);
    if(!data->getComment().empty())
        query += "', Comment = '" + maskChars(data->getComment());
    if(data->getRating() > 0)
        query += "', Rating = '" + iToStr(data->getRating());
    query += "' WHERE ID = '" + iToStr(data->getId()) + "'";
    
    this->getMMSDBConnection()->query(query);
}

MMSPhotoData MMSPhotoDAO::moveRecordToData(MMSRecordSet &rs) {
    MMSPhotoData data;

    data.setId(atoi(rs["ID"].c_str()));
    data.setFilename(rs["Filename"]);
    data.setTitle(rs["Title"]);
    data.setDate(rs["Date"]);
    data.setComment(rs["Comment"]);
    data.setRating(atoi(rs["Rating"].c_str()));
    data.setPlace(rs["PhotoPlace.Name"]);
    data.setCamera(rs["PhotoCamera.Name"]);
    data.setAlbum(rs["PhotoAlbum.Name"]);
    data.setAlbumCover(rs["PhotoAlbum.Cover"]);
    data.setFormat(rs["PhotoFormat.Name"]);
    
    return data;
}

MMSPhotoDAO::MMSPhotoDAO(IMMSDB *connection) {
    setMMSDBConnection(connection);
}

void MMSPhotoDAO::save(MMSPhotoData *data) {
    /* check if ID is set */
    if (data->getId()<0)
        /* no, have to insert */
        insert(data);
    else
        /* yes, have to update */
        update(data);

}

void MMSPhotoDAO::save(vector<MMSPhotoData *> dataList) {
    /* for each item */
    for (unsigned i=0;i<dataList.size();i++) {
        /* check if ID is set */
        if (dataList.at(i)->getId()<0)
            /* no, have to insert */
            insert(dataList.at(i));
        else
            /* yes, have to update */
            update(dataList.at(i));
    }
}

void MMSPhotoDAO::getStatistics(unsigned int &numPhotos, unsigned int &numPlaces, unsigned int &numAlbums) {
    MMSRecordSet rs; 

    numPhotos  = 0;
    numPlaces = 0;
    numAlbums = 0;

    this->getMMSDBConnection()->query("SELECT PhotoCount, PlaceCount, AlbumCount FROM PhotoStatistics WHERE ID = '0'", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return;

    /* set the values */
    numPhotos = atoi(rs["PhotoCount"].c_str());
    numPlaces = atoi(rs["PlaceCount"].c_str());
    numAlbums = atoi(rs["AlbumCount"].c_str());
}

void MMSPhotoDAO::getStatistics(unsigned int &numPhotos, unsigned int &numPlaces, unsigned int &numAlbums, string &lastViewed) {
    MMSRecordSet rs; 

    numPhotos   = 0;
    numPlaces  = 0;
    numAlbums  = 0;
    lastViewed = "";

    this->getMMSDBConnection()->query("SELECT PhotoCount, PlaceCount, AlbumCount, LastViewed FROM PhotoStatistics WHERE ID = '0'", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return;

    /* set the values */
    numPhotos  = atoi(rs["PhotoCount"].c_str());
    numPlaces  = atoi(rs["PlaceCount"].c_str());
    numAlbums  = atoi(rs["AlbumCount"].c_str());
    lastViewed = rs["LastViewed"];
}

void MMSPhotoDAO::setLastViewed(const string lastViewed) {
    string query = "UPDATE PhotoStatistics ";
    query += "SET LastViewed = '" + maskChars(lastViewed);
    query += "' WHERE ID = '0'";
    
    this->getMMSDBConnection()->query(query);
}

vector <MMSPhotoData *> MMSPhotoDAO::getPhotos() {
    vector <MMSPhotoData *> ret;
    MMSRecordSet    rs;

    string query = "SELECT Filename, \
                           Title, \
                           Date, \
                           Comment, \
                           Rating, \
                           PhotoPlace.Name, \
                           PhotoCamera.Name, \
                           PhotoAlbum.Name, \
                           PhotoAlbum.Cover, \
                           PhotoFormat.Name, \
                    FROM   PhotoFile, \
                           PhotoPlace, \
                           PhotoCamera, \
                           PhotoAlbum, \
                           PhotoFormat \
                    WHERE  PhotoFile.IDPlace  = PhotoPlace.ID \
                    AND    PhotoFile.IDCamera = PhotoCamera.ID \
                    AND    PhotoFile.IDAlbum  = PhotoAlbum.ID \
                    AND    PhotoFile.IDFormat = PhotoFormat.ID";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        MMSPhotoData *data = new MMSPhotoData;
        *data = moveRecordToData(rs);

        /* push to list */
        ret.push_back(data);
    }
    while(rs.next() == true);
     
    return ret;
}

vector <MMSPhotoData *> MMSPhotoDAO::getPhotosByAlbum(const string album) {
    vector <MMSPhotoData *> ret;
    MMSRecordSet    rs; 

    string query = "SELECT Filename, \
                           Title, \
                           Date, \
                           Comment, \
                           Rating, \
                           PhotoPlace.Name, \
                           PhotoCamera.Name, \
                           PhotoAlbum.Name, \
                           PhotoAlbum.Cover, \
                           PhotoFormat.Name \
                    FROM   PhotoFile, \
                           PhotoPlace, \
                           PhotoCamera, \
                           PhotoAlbum, \
                           PhotoFormat \
                    WHERE  PhotoAlbum.Name = '" + maskChars(album) + "' \
                    AND    PhotoFile.IDPlace  = PhotoPlace.ID \
                    AND    PhotoFile.IDCamera = PhotoCamera.ID \
                    AND    PhotoFile.IDAlbum  = PhotoAlbum.ID \
                    AND    PhotoFile.IDFormat = PhotoFormat.ID";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        MMSPhotoData *data = new MMSPhotoData;
        *data = moveRecordToData(rs);

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

vector <MMSPhotoData *> MMSPhotoDAO::getPhotosByPlace(const string place) {
    vector <MMSPhotoData *> ret;
    MMSRecordSet    rs; 

    string query = "SELECT Filename, \
                           Title, \
                           Date, \
                           Comment, \
                           Rating, \
                           PhotoPlace.Name, \
                           PhotoCamera.Name, \
                           PhotoAlbum.Name, \
                           PhotoAlbum.Cover, \
                           PhotoFormat.Name \
                    FROM   PhotoFile, \
                           PhotoPlace, \
                           PhotoCamera, \
                           PhotoAlbum, \
                           PhotoFormat \
                    WHERE  PhotoPlace.Name = '" + maskChars(place) + "' \
                    AND    PhotoFile.IDPlace  = PhotoPlace.ID \
                    AND    PhotoFile.IDCamera = PhotoCamera.ID \
                    AND    PhotoFile.IDAlbum  = PhotoAlbum.ID \
                    AND    PhotoFile.IDFormat = PhotoFormat.ID";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        MMSPhotoData *data = new MMSPhotoData;
        *data = moveRecordToData(rs);

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

vector < pair<string, string> > MMSPhotoDAO::getAlbums() {
    vector < pair<string, string> > ret;
    MMSRecordSet    rs; 

    this->getMMSDBConnection()->query("SELECT PhotoAlbum.Name, PhotoAlbum.Cover FROM PhotoAlbum GROUP BY PhotoAlbum.Name", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        string name = rs["PhotoAlbum.Name"];
        string cover = rs["PhotoAlbum.Cover"];

        /* push to list */
        ret.push_back(make_pair(name, cover));
    } 
    while(rs.next() == true);
     
    return ret;
}

vector <string> MMSPhotoDAO::getPlaces() {
    vector <string> ret;
    MMSRecordSet    rs; 

    this->getMMSDBConnection()->query("SELECT Places FROM PhotoPlace GROUP BY PhotoPlace.Name", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        string data = rs["PhotoPlace.Name"];

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

MMSPhotoData *MMSPhotoDAO::getPhotoByFilename(const string filename) {
    MMSPhotoData *ret = NULL;
    MMSRecordSet    rs; 
    
    string query = "SELECT Filename, \
                           Title, \
                           Date, \
                           Comment, \
                           Rating, \
                           PhotoPlace.Name, \
                           PhotoCamera.Name, \
                           PhotoAlbum.Name, \
                           PhotoAlbum.Cover, \
                           PhotoFormat.Name \
                    FROM   PhotoFile, \
                           PhotoPlace, \
                           PhotoCamera, \
                           PhotoAlbum, \
                           PhotoFormat \
                    WHERE  Filename = '" + maskChars(filename) + "' \
                    AND    PhotoFile.IDPlace  = PhotoPlace.ID \
                    AND    PhotoFile.IDCamera = PhotoCamera.ID \
                    AND    PhotoFile.IDAlbum  = PhotoAlbum.ID \
                    AND    PhotoFile.IDFormat = PhotoFormat.ID";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;
    ret = new MMSPhotoData();
    
    *ret = moveRecordToData(rs);
    
    return ret;
}

bool MMSPhotoDAO::albumExists(const string album) {
    MMSRecordSet    rs; 

    string query = "SELECT ID FROM PhotoAlbum WHERE Name = '" + maskChars(album) + "'";
    this->getMMSDBConnection()->query(query, &rs);

    return (rs.getCount() > 0);
}
