/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *      Matthias Hardt   <mattmax@morphine.tv>                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mmsdata/mmsphotoservice.h"

MMSPhotoService::MMSPhotoService(DataSource *datasource) :
    dbconn(NULL) {
    MMSDBConnMgr connMgr(datasource); 
    if((this->dbconn = connMgr.getConnection()))
        this->dbconn->connect();
}

MMSPhotoService::~MMSPhotoService() {
	if(this->dbconn) {
        this->dbconn->disconnect();
        delete this->dbconn;
	}
}

void MMSPhotoService::setPhotoData(MMSPhotoData *data) {
    MMSPhotoDAO dao(this->dbconn);
    dao.save(data);
}

void MMSPhotoService::setPhotoData(vector<MMSPhotoData *> dataList) {
    MMSPhotoDAO dao(this->dbconn);
    dao.save(dataList);
}

void MMSPhotoService::getStatistics(unsigned int &numPhotos, unsigned int &numPlaces, unsigned int &numAlbums) {
    MMSPhotoDAO dao(this->dbconn);
    dao.getStatistics(numPhotos, numPlaces, numAlbums);
}

void MMSPhotoService::getStatistics(unsigned int &numPhotos, unsigned int &numPlaces, unsigned int &numAlbums, string &lastViewed) {
    MMSPhotoDAO dao(this->dbconn);
    dao.getStatistics(numPhotos, numPlaces, numAlbums, lastViewed);
}

void MMSPhotoService::setLastViewed(const string lastViewed) {
    MMSPhotoDAO dao(this->dbconn);
    dao.setLastViewed(lastViewed);
}

vector <string> MMSPhotoService::getPlaces() {
    MMSPhotoDAO dao(this->dbconn);
    return dao.getPlaces();
}

vector < pair<string, string> > MMSPhotoService::getAlbums() {
    MMSPhotoDAO dao(this->dbconn);
    return dao.getAlbums();
}

MMSPhotoData *MMSPhotoService::getPhotoByFilename(const string filename) {
    MMSPhotoDAO dao(this->dbconn);
    return dao.getPhotoByFilename(filename);
}

vector <MMSPhotoData *> MMSPhotoService::getPhotos() {
    MMSPhotoDAO dao(this->dbconn);
    return dao.getPhotos();
}

vector <MMSPhotoData *> MMSPhotoService::getPhotosByPlace(const string place) {
    MMSPhotoDAO dao(this->dbconn);
    return dao.getPhotosByPlace(place);
}

vector <MMSPhotoData *> MMSPhotoService::getPhotosByAlbum(const string album) {
    MMSPhotoDAO dao(this->dbconn);
    return dao.getPhotosByAlbum(album);
}

bool MMSPhotoService::albumExists(const string album) {
    MMSPhotoDAO dao(this->dbconn);
    return dao.albumExists(album);
}
