/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mmsdata/mmstvchanneldao.h"

MMSTVChannelDAO::MMSTVChannelDAO(IMMSDB *connection) {
    setMMSDBConnection(connection);
}

void MMSTVChannelDAO::setMMSDBConnection(IMMSDB *connection) {
    this->dbConnection = connection;
}

IMMSDB *MMSTVChannelDAO::getMMSDBConnection() {
    return this->dbConnection;   
}

void MMSTVChannelDAO::insert(MMSTVChannelData *data) {
    /* do the insert */
    this->getMMSDBConnection()->query(
        "insert into TVChannel(ImportSourceID,Number,Name,SID,Line,TVID,BouquetID) \
        values('"
        + iToStr(data->getSourceId()) + "','"
        + iToStr(data->getNumber()) + "','"
        + maskChars(data->getName()) + "','"
        + iToStr(data->getSID()) + "','"
        + maskChars(data->getLine()) + "','"
        + iToStr(data->getTVID()) + "','"
        + iToStr(data->getBouquetID()) + "')");

    /* set the ID */
    data->setId(this->getMMSDBConnection()->getLastInsertedID());
}

void MMSTVChannelDAO::update(MMSTVChannelData *data) {
    /* do the update */
    this->getMMSDBConnection()->query(
        "update TVChannel set Number='" + iToStr(data->getNumber()) + "',"
        + "Name='" + maskChars(data->getName()) + "',"
        + "SID='" + iToStr(data->getSID()) + "',"
        + "Line='" + maskChars(data->getLine()) + "' "
        "where ID = '" + iToStr(data->getId()) + "'");
}

void MMSTVChannelDAO::save(MMSTVChannelData *data) {
    /* check if ID is set */
    if (data->getId()<0)
        /* no, have to insert */
        insert(data);
    else
        /* yes, have to update */
        update(data);
}

void MMSTVChannelDAO::save(vector<MMSTVChannelData *> dataList) {
    /* for each item */
    for (unsigned i=0;i<dataList.size();i++) {
        /* check if ID is set */
        if (dataList.at(i)->getId()<0)
            /* no, have to insert */
            insert(dataList.at(i));
        else
            /* yes, have to update */
            update(dataList.at(i));
    }
}

MMSTVChannelData MMSTVChannelDAO::moveRecordToData(MMSRecordSet &rs) {
    MMSTVChannelData data;

    data.setId(atoi(rs["ID"].c_str()));
    data.setSourceId(atoi(rs["ImportSourceID"].c_str()));
    data.setNumber(atoi(rs["Number"].c_str()));
    data.setName(rs["Name"]);
    data.setSID(atoi(rs["SID"].c_str()));
    data.setLine(rs["Line"]);
    data.setTVID(atoi(rs["TVID"].c_str()));
    data.setBouquetID(atoi(rs["BouquetID"].c_str()));
    
    return data;
}


vector<MMSTVChannelData *> MMSTVChannelDAO::findAllChannels() {
    vector<MMSTVChannelData *>  list;
    MMSRecordSet                rs;

    /* do query */
    this->getMMSDBConnection()->query("select * from TVChannel", &rs);

    /* check if result is empty */
    if (rs.getCount()==0) return list;

    /* for each result record */
    do {
        /* set the values */
        MMSTVChannelData *data = new MMSTVChannelData;
        *data = moveRecordToData(rs);

        /* push to list */
        list.push_back(data);
    } while(rs.next() == true);
        
    return list;
}

MMSTVChannelData *MMSTVChannelDAO::findChannelByName(string name) {
    MMSTVChannelData    *data;
    MMSRecordSet        rs;

    /* do query */
    this->getMMSDBConnection()->query("select * from TVChannel where Name = '" + maskChars(name) + "'", &rs);

    /* check if result is empty */
    if (rs.getCount()==0)
        return NULL;

    /* set the values */
    data = new MMSTVChannelData;
    *data = moveRecordToData(rs);
    return data;
}
