/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *      Matthias Hardt   <mattmax@morphine.tv>                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mmsdata/mmsvideodao.h"

void MMSVideoDAO::setMMSDBConnection(IMMSDB *connection) {
    this->dbConnection = connection;
}

IMMSDB *MMSVideoDAO::getMMSDBConnection() {
    return this->dbConnection;   
}

unsigned int MMSVideoDAO::getGenreID(const string genre, const bool update) {
    MMSRecordSet rs;
    
    if(!genre.empty()) {
        // check if genre is already present
        try {
            this->getMMSDBConnection()->query("SELECT ID FROM VideoGenre WHERE Name = '" + maskChars(genre) + "'", &rs);
            if(!rs["ID"].empty())
                return strToInt(rs["ID"]);
        }
        catch(MMSError *e) {
        	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
            return 0;
        }

        if(update) {
            try {
                string query = "INSERT INTO VideoGenre(Name) VALUES('";
                query += maskChars(genre);
                query += "')";
                this->getMMSDBConnection()->query(query);
                this->getMMSDBConnection()->query("SELECT ID FROM VideoGenre WHERE Name = '" + maskChars(genre) + "'", &rs);
                return strToInt(rs["ID"]);
            }
            catch(MMSError *e) {
            	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
                return 0;
            }
        }
    }

    return 0;
}

unsigned int MMSVideoDAO::getFormatID(const string format, const bool update) {
    MMSRecordSet rs;
    
    if(!format.empty()) {
        // check if format is already present
        try {
            this->getMMSDBConnection()->query("SELECT ID FROM VideoFormat WHERE Name = '" + maskChars(format) + "'", &rs);
            if(!rs["ID"].empty())
                return strToInt(rs["ID"]);
        }
        catch(MMSError *e) {
        	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
            return 0;
        }

        if(update) {
            try {
                this->getMMSDBConnection()->query("INSERT INTO VideoFormat(Name) VALUES('" + maskChars(format) + "')");
                this->getMMSDBConnection()->query("SELECT ID FROM VideoFormat WHERE Name = '" + maskChars(format) + "'", &rs);
                return strToInt(rs["ID"]);
            }
            catch(MMSError *e) {
            	DEBUGMSG("MMSMusicDAO", "Error: %s", e->getMessage().c_str());
                return 0;
            }
        }
    }

    return 0;
}

/**
 * Inserts the video data into the corresponding
 * tables.
 * 
 * 
 */
void MMSVideoDAO::insert(MMSVideoData *data) {
    unsigned int idGenre, idFormat;
    string query; 

    try {
        idGenre  = this->getGenreID(data->getGenre());
        idFormat = this->getFormatID(data->getFormat());
        if(data->getTitle().empty()) {
            size_t pos = data->getFilename().rfind("/");
            data->setTitle(data->getFilename().substr(((pos != string::npos) ? pos + 1 : 0), string::npos));
        }
        
        query = "INSERT INTO VideoFile(Filename, Title, IDGenre, IDFormat";
        if(!data->getCover().empty())
            query += ", Cover";
        if(!data->getDirector().empty())
            query += ", Director";
        if(!data->getRelease().empty())
            query += ", Release";
        if(!data->getComment().empty())
            query += ", Comment";
        if(data->getRating() > 0)
            query += ", Rating";
        query += ") Values('";
        query += maskChars(data->getFilename());
        query += "', '" + maskChars(data->getTitle());
        query += "', '" + iToStr(idGenre);
        query += "', '" + iToStr(idFormat);
        if(!data->getCover().empty())
            query += "', '" + maskChars(data->getCover());
        if(!data->getDirector().empty())
            query += "', '" + maskChars(data->getDirector());
        if(!data->getRelease().empty())
            query += "', '" + maskChars(data->getRelease());
        if(!data->getComment().empty())
            query += "', '" + maskChars(data->getComment());
        if(data->getRating() > 0)
            query += "', '" + iToStr(data->getRating());
        query += "')";
        
        this->getMMSDBConnection()->query(query);
    } catch(MMSError *error) {
        /* failed to insert, try update */
        update(data);
        return;
    }

    /* set the ID */
    data->setId(this->getMMSDBConnection()->getLastInsertedID());

}

void MMSVideoDAO::update(MMSVideoData *data) {
    unsigned int idGenre, idFormat;
    string query; 

    idGenre  = this->getGenreID(data->getGenre());
    idFormat = this->getFormatID(data->getFormat());

    query = "UPDATE VideoFile ";
    query += "SET Filename = '" + maskChars(data->getFilename());
    query += "', Title = '" + maskChars(data->getTitle());
    query += "', IDGenre = '" + iToStr(idGenre);
    query += "', IDFormat = '" + iToStr(idFormat);
    if(!data->getCover().empty())
        query += "', Cover = '" + maskChars(data->getCover());
    if(!data->getDirector().empty())
        query += "', Director = '" + maskChars(data->getDirector());
    if(!data->getRelease().empty())
        query += "', Release = '" + maskChars(data->getRelease());
    if(!data->getComment().empty())
        query += "', Comment = '" + maskChars(data->getComment());
    if(data->getRating() > 0)
        query += "', Rating = '" + iToStr(data->getRating());
    query += "' WHERE ID = '" + iToStr(data->getId()) + "'";
    
    this->getMMSDBConnection()->query(query);
}

MMSVideoData MMSVideoDAO::moveRecordToData(MMSRecordSet &rs) {
    MMSVideoData data;

    data.setId(atoi(rs["ID"].c_str()));
    data.setFilename(rs["Filename"]);
    data.setTitle(rs["Title"]);
    data.setCover(rs["Cover"]);
    data.setDirector(rs["Director"]);
    data.setRelease(rs["Release"]);
    data.setComment(rs["Comment"]);
    data.setRating(atoi(rs["Rating"].c_str()));
    data.setGenre(rs["VideoGenre.Name"]);
    data.setFormat(rs["VideoFormat.Name"]);
    
    return data;
}

MMSVideoDAO::MMSVideoDAO(IMMSDB *connection) {
    setMMSDBConnection(connection);
}

void MMSVideoDAO::save(MMSVideoData *data) {
    /* check if ID is set */
    if (data->getId()<0)
        /* no, have to insert */
        insert(data);
    else
        /* yes, have to update */
        update(data);

}

void MMSVideoDAO::save(vector<MMSVideoData *> dataList) {
    /* for each item */
    for (unsigned i=0;i<dataList.size();i++) {
        /* check if ID is set */
        if (dataList.at(i)->getId()<0)
            /* no, have to insert */
            insert(dataList.at(i));
        else
            /* yes, have to update */
            update(dataList.at(i));
    }
}

void MMSVideoDAO::getStatistics(unsigned int &numVideos, unsigned int &numGenres) {
    MMSRecordSet rs; 

    numVideos = 0;
    numGenres = 0;

    this->getMMSDBConnection()->query("SELECT VideoCount, GenreCount FROM VideoStatistics WHERE ID = '0'", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return;

    /* set the values */
    numVideos  = atoi(rs["VideoCount"].c_str());
    numGenres  = atoi(rs["GenreCount"].c_str());
}

void MMSVideoDAO::getStatistics(unsigned int &numVideos, unsigned int &numGenres, string &lastPlayed) {
    MMSRecordSet rs; 

    numVideos = 0;
    numGenres = 0;
    lastPlayed = "";

    this->getMMSDBConnection()->query("SELECT VideoCount, GenreCount, LastPlayed FROM VideoStatistics WHERE ID = '0'", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return;

    /* set the values */
    numVideos  = atoi(rs["VideoCount"].c_str());
    numGenres  = atoi(rs["GenreCount"].c_str());
    lastPlayed = rs["LastPlayed"];
}

void MMSVideoDAO::setLastPlayed(const string lastPlayed) {
    string query = "UPDATE VideoStatistics ";
    query += "SET LastPlayed = '" + maskChars(lastPlayed);
    query += "' WHERE ID = '0'";
    
    this->getMMSDBConnection()->query(query);
}

vector <MMSVideoData *> MMSVideoDAO::getVideos() {
    vector <MMSVideoData *> ret;
    MMSRecordSet    rs;

    string query = "SELECT Filename, \
                           Title, \
                           Cover, \
                           Director, \
                           Release, \
                           Comment, \
                           Rating, \
                           VideoGenre.Name, \
                           VideoFormat.Name \
                    FROM   VideoFile, \
                           VideoGenre, \
                           VideoFormat \
                    WHERE  VideoFile.IDGenre  = VideoGenre.ID \
                    AND    VideoFile.IDFormat = VideoFormat.ID \
                    ORDER BY Title";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        MMSVideoData *data = new MMSVideoData;
        *data = moveRecordToData(rs);

        /* push to list */
        ret.push_back(data);
    }
    while(rs.next() == true);
     
    return ret;
}

vector <MMSVideoData *> MMSVideoDAO::getVideosByGenre(string genre) {
    vector <MMSVideoData *> ret;
    MMSRecordSet    rs; 

    string query = "SELECT Filename, \
                           Title, \
                           Cover, \
                           Director, \
                           Release, \
                           Comment, \
                           Rating, \
                           VideoGenre.Name, \
                           VideoFormat.Name \
                    FROM   VideoFile, \
                           VideoGenre, \
                           VideoFormat \
                    WHERE  VideoGenre.Name = '" + maskChars(genre) + "' \
                    AND    VideoFile.IDFormat = VideoFormat.ID \
                    ORDER BY Title";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        MMSVideoData *data = new MMSVideoData;
        *data = moveRecordToData(rs);

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

vector <string *> MMSVideoDAO::getGenres() {
    vector <string *> ret;
    MMSRecordSet    rs; 

    this->getMMSDBConnection()->query("SELECT VideoGenre.Name FROM VideoGenre GROUP BY VideoGenre.Name ORDER BY VideoGenre.Name", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        string *data = new string;
        *data = rs["VideoGenre.Name"];

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

vector <string *> MMSVideoDAO::getFormats() {
    vector <string *> ret;
    MMSRecordSet    rs; 

    this->getMMSDBConnection()->query("SELECT Description FROM VideoFormat", &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;

    /* for each result record */
    do {
        /* set the values */
        string *data = new string;
        *data = rs["Description"];

        /* push to list */
        ret.push_back(data);
    } 
    while(rs.next() == true);
     
    return ret;
}

MMSVideoData *MMSVideoDAO::getVideoByFilename(string filename) {
    MMSVideoData *ret = NULL;
    MMSRecordSet    rs; 
    
    string query = "SELECT Filename, \
                           Title, \
                           Cover, \
                           Director, \
                           Release, \
                           Comment, \
                           Rating, \
                           VideoGenre.Name, \
                           VideoFormat.Name \
                    FROM   VideoFile, \
                           VideoGenre, \
                           VideoFormat \
                    WHERE  Filename = '" + maskChars(filename) + "' \
                    AND    VideoFile.IDGenre  = VideoGenre.ID \
                    AND    VideoFile.IDFormat = VideoFormat.ID";
    this->getMMSDBConnection()->query(query, &rs);

    /* check if result is empty */
    if(rs.getCount() == 0) return ret;
    ret = new MMSVideoData();
    
    *ret = moveRecordToData(rs);
    
    return ret;
}
