/***************************************************************************
 *   Copyright (C) 2005-2007 by                                            *
 *                                                                         *
 *      Stefan Schwarzer <sxs@morphine.tv>                                 *
 *      Guido Madaus     <bere@morphine.tv>                                *
 *      Jens Schneider   <pupeider@morphine.tv>                            *
 *      Matthias Hardt   <mattmax@morphine.tv>                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mmsdata/mmsvideoservice.h"

MMSVideoService::MMSVideoService(DataSource *datasource) :
    dbconn(NULL) {
    MMSDBConnMgr connMgr(datasource); 
    if((this->dbconn = connMgr.getConnection()))
    	this->dbconn->connect();
}

MMSVideoService::~MMSVideoService() {
	if(this->dbconn) {
		this->dbconn->disconnect();
		delete this->dbconn;
	}
}

void MMSVideoService::setVideoData(MMSVideoData *data) {
    MMSVideoDAO dao(this->dbconn);
    dao.save(data);

}

void MMSVideoService::setVideoData(vector<MMSVideoData *> dataList) {
    MMSVideoDAO dao(this->dbconn);
    dao.save(dataList);
}

void MMSVideoService::getStatistics(unsigned int &numVideos, unsigned int &numGenres) {
    MMSVideoDAO dao(this->dbconn);
    dao.getStatistics(numVideos, numGenres);
}

void MMSVideoService::getStatistics(unsigned int &numVideos, unsigned int &numGenres, string &lastPlayed) {
    MMSVideoDAO dao(this->dbconn);
    dao.getStatistics(numVideos, numGenres, lastPlayed);
}

void MMSVideoService::setLastPlayed(const string lastPlayed) {
    MMSVideoDAO dao(this->dbconn);
    dao.setLastPlayed(lastPlayed);
}

vector <string *> MMSVideoService::getGenres() {
    MMSVideoDAO dao(this->dbconn);
    return dao.getGenres();
}

vector <string *> MMSVideoService::getFormats() {
    MMSVideoDAO dao(this->dbconn);
    return dao.getFormats();
}

MMSVideoData *MMSVideoService::getVideoByFilename(string filename) {
    MMSVideoDAO dao(this->dbconn);
    return dao.getVideoByFilename(filename);
}

vector <MMSVideoData *> MMSVideoService::getVideos() {
    MMSVideoDAO dao(this->dbconn);
    return dao.getVideos();
}

vector <MMSVideoData *> MMSVideoService::getVideosByGenre(string genre) {
    MMSVideoDAO dao(this->dbconn);
    return dao.getVideosByGenre(genre);
}
